# -*- coding: utf-8 -*-
"""
Created on Fry Jan 1 12:33:57 2021

@author: Klaus Reinold
"""

from graphics_and_games_klassen import Rechteck, Kreis, Text, Ereignisbehandlung
import random  



class FahrzeugSymbol:
    """
    Ein Fahrzeugsymbol
    """



    def __init__(self):
        """
        StandardInitialisierung für Objekte der Klasse FahrzeugSymbol.
        """    
        self.laenge = 22
        self.breite = 9
        self.aktFarbe = "blau"
        self.aufbau = Rechteck()
        self.aufbau.PositionSetzen(0, 0)
        self.aufbau.GroesseSetzen(self.breite, self.laenge)
        self.aufbau.FarbeSetzen(self.aktFarbe)
        self.scheibe = Rechteck()
        self.scheibe.PositionSetzen(1, self.laenge/4)
        self.scheibe.GroesseSetzen(self.breite - 2, self.laenge/4)
        self.scheibe.FarbeSetzen("weiß")
    


    def FarbeSetzen(self, neueFarbe):
        """
        Setzt die Farbe des Fahrzeugsymbols.
        -- Parameter neueFarbe die neue Farbe des Fahrzeugsymbols
        """    
        self.aktFarbe = neueFarbe
        self.aufbau.FarbeSetzen(self.aktFarbe)
    


    def PositionSetzen(self, x, y):
        """
        Setzt die Position des Fahrzeugsymbols.
        Die Koordinaten bezeichnen die linke, obere Ecke des Fahrzeugsymbols.
        -- Parameter x x-Position
        -- Parameter y y-Position
        """    
        self.x = x
        self.y = y
        self.aufbau.PositionSetzen(x, y)
        self.scheibe.PositionSetzen(x + 1, y + self.laenge/4)
    


    def Entfernen(self):
        """
        Entfernt das Fahrzeugsymbol aus der Anzeige
        """    
        self.aufbau.Entfernen()
        self.scheibe.Entfernen()
    





class FahrbahnSymbol:
    """
    Stellt die Fahrbahn auf dem Ausgabefenster dar.
    """



    def __init__(self):
        """
        Erzeugt das FahrbahnSymbol
        """    
        self.x = 0
        self.y = 0
        self.hintergrund = Rechteck()
        self.hintergrund.PositionSetzen(self.x, self.y)
        self.hintergrund.GroesseSetzen(20, 600)
        self.hintergrund.FarbeSetzen("grau")
        self.haltelinie = Rechteck()
        self.haltelinie.PositionSetzen(self.x + 3, self.y + 200)
        self.haltelinie.GroesseSetzen(17, 3)
        self.haltelinie.FarbeSetzen("weiß")
    


    def PositionSetzen(self, x, y):
        """
        Setzt die Position der Fahrbahn.
        -- Parameter x x-Position des Kreuzungsmittelpunkts
        -- Parameter y y-Position des Kreuzungsmittelpunkts
        """     
        self.x = x
        self.y = y
        self.hintergrund.PositionSetzen(self.x, self.y)
        self.haltelinie.PositionSetzen(self.x + 3, self.y + 200)
    


    def EintrittsPositionGeben(self):
        """
        Meldet die Eintrittsposition des Fahrzeugs
        -- return Startposition des Fahrzeugs
        """    
        return self.y + 600
    


    def AustrittsPositionGeben(self):
        """
        Meldet die Austrittsposition des Fahrzeugs
        -- return Austrittsposition des Fahrzeugs
        """   
        return self.y
    


    def RandAbstandGeben(self):
        """
        Meldet den Abstand zum inneren Fahrbahnrand
        -- return Abstand des Fahrzeugs
        """    
        return 5
    


    def HaltelinienPositionGeben(self):
        """
        Meldet die Position der Haltelinie
        -- return Haltelinienposition
        """    
        return self.y + 205
    





class Lampe:
    """
    Beschreibt eine Lampe für eine Ampel
    """

    

    def __init__(self, x, y, farbe):
        """
        Legt eine Lampe an der gegebenen Position an.
        -- Parameter x x-Koordinate der Lampe
        -- Parameter y y-Koordinate der Lampe
        -- Parameter farbe Farbe der Lampe
        """    
        self.rahmen = Rechteck()
        self.rahmen.PositionSetzen(x, y)
        self.rahmen.GroesseSetzen(20, 20)
        self.rahmen.FarbeSetzen("schwarz")
        self.flaeche = Rechteck()
        self.flaeche.PositionSetzen(x + 1, y + 1)
        self.flaeche.GroesseSetzen(18, 18)
        self.flaeche.FarbeSetzen("weiß")
        self.anzeige = Kreis()
        self.anzeige.RadiusSetzen(5)
        self.anzeige.PositionSetzen(x + 10, y + 10)
        self.anzeige.FarbeSetzen(farbe)
        self.farbe = farbe
    
    

    def Einschalten(self):
        """
        Schaltet die Lampe ein
        """    
        self.anzeige.FarbeSetzen(self.farbe)
    
    

    def Ausschalten(self):
        """
        Schaltet die Lampe aus
        """    
        self.anzeige.FarbeSetzen("schwarz")
    
    

    def PositionSetzen(self, x, y):
        """
        Setzt die Position der Lampe
        -- Parameter x x-Koordinate der Lampe
        -- Parameter y y-Koordinate der Lampe
        """    
        self.rahmen.PositionSetzen(x, y)
        self.flaeche.PositionSetzen(x + 1, y + 1)
        self.anzeige.PositionSetzen(x + 10, y + 10)
    




class Fahrzeug:
    """
    Beschreibt ein Fahrzeug.
    """



    def __init__(self):
        """
        Erzeugt die Anzeige und setzt die Startdaten
        """    
        self.vmax = 13.89
        self.v = self.vmax
        self.a = 4.0
        self.x = 0
        self.y = 0
        self.symbol = FahrzeugSymbol()
        self.symbol.FarbeSetzen(self.ZufallsfarbeErzeugen())
        self.symbol.PositionSetzen(self.x, self.y)
    


    def ZufallsfarbeErzeugen(self):
        """
        Erzeugt eine zufällige Farbe für das Fahrzeug
        """    
        zz = random.randint(0, 6)
        if zz == 0:
            return "rot"
        elif zz== 1:
            return "grün"
        elif zz== 2:
            return "blau"
        elif zz== 3:
            return "gelb"
        elif zz== 4:
            return "cyan"
        elif zz== 5:
            return "magenta"
        elif zz== 6:
            return "weiss"
        else:
            return ""
        
    


    def PositionSetzen(self, x, y):
        """
        Setzt die Position des Fahrzeugs.
        
        -- Parameter x x-Position
        -- Parameter y y-Position
        """    
        self.x = x
        self.y = y
        self.symbol.PositionSetzen (self.x, self.y)
    
    

    def Fahren(self, wegmax, vvorg):
        """
        Fährt das Fahrzeug eine Sekunde nach oben.
        -- Parameter wegmax maximaler Weg bis zum nächsten Hindernis (Ampel oder VOrgänger):
        -- Parameter vvorg Geschwindigkeit des Vorgängerfahrzeugs bzw.der Ampel
        """    

        weg = 0
        for i in range(0, 10):            # Maximaler Weg
            vneu = self.a*0.1 + self.v
            if vneu > self.vmax: 
                vneu = self.vmax      
            weg = weg + (vneu + self.v)/ 2.0*0.1
            self.v = vneu  
        weg = weg*5                         # 1Px = 20cm
        if (wegmax <= 0.0):                     # Muss stehen bleiben
            weg = 0.0
            self.v = 0.0
        elif weg >= wegmax:                 # Kommt zu dicht an den Vorgänger
            weg = wegmax
            if self.v > vvorg:                      # und ist auch noch zu schnell     
                self.v = vvorg
        else:        
            vneu = (wegmax / 5.0 + vvorg*1.0 - weg / 5.0) / 1.8 # Maximale Geschwindigkeit für den Restabstand zum Vorgänger
            if (vneu < self.v):                       # Zu schnell für den Abstand            
                self.v = vneu  
        self.PositionSetzen (self.x, self.y - int(weg))
            


    def XKoordinateGeben(self):
        """
        Meldet die x-Koordinate zurück
        -- return x-Koordinate
        """    
        return self.x
    


    def YKoordinateGeben(self):
        """
        Meldet die y-Koordinate zurück
        -- return y-Koordinate
        """    
        return self.y
    


    def LaengeGeben(self):
        """
        Meldet die Länge des Fahrzeugs zurück
        -- return Länge
        """    
        return 22
    


    def GeschwindigkeitGeben(self):
        """
        Meldet die Geschwindigkeit zurück
        -- return Geschwindigkeit
        """   
        return self.v
    


    def Entfernen(self):
        """
        Entfernt das Fahrzeug aus der Anzeige
        """    
        self.symbol.Entfernen()
    



class Ampel(Ereignisbehandlung):
    """
    Eine Ampel verwendet zur Anzeige drei Lampen.
    Die Ampel verwaltet ihre Position als Ganzes, ihre Ausrichtung und auch die 
    nach den Verkehrsregeln möglichen Ampelphasen.
    Mögliche Werte sind die in Deutschland erlaubten Ampelphasen 
    grün, gelb, rot und rotgelb.
    """


    def __init__(self):
        """
        Initialisierung für Objekte der Klasse Ampel.
        """    
        self(100, 100)
    
   

    def __init__(self, x, y):
        """
        Initialisierung für Objekte der Klasse Ampel 
        mit folgenden Eingabewerten.
        -- Parameter x - x-Position
        -- Parameter y - y-Position
        """ 
        super().__init__()
        self.ampelphase = "rot"
        self.wartezeit = 0
       
        self.lampeOben = Lampe(x, y, "rot")
        self.lampeMitte = Lampe(x, y + 20, "gelb")
        self.lampeUnten = Lampe(x, y + 40, "grün")
        self.lampeOben.Einschalten()
        self.lampeMitte.Ausschalten()
        self.lampeUnten.Ausschalten()
    
           

    def GruenSetzen(self):
        """
        Setzt die Ampel auf grün.
        """    
        self.ampelphase = "grün"
        self.lampeOben.Ausschalten()
        self.lampeMitte.Ausschalten()
        self.lampeUnten.Einschalten()
    
    

    def GelbSetzen(self):
        """
        Setzt die Ampel auf gelb.
        """    
        self.ampelphase = "gelb"
        self.lampeOben.Ausschalten()
        self.lampeMitte.Einschalten()
        self.lampeUnten.Ausschalten()
    

    def RotgelbSetzen(self):
        """
        Setzt die Ampel auf rot-gelb.
        """   
        self.ampelphase = "rotgelb"
        self.lampeOben.Einschalten()
        self.lampeMitte.Einschalten()
        self.lampeUnten.Ausschalten()
    
    

    def RotSetzen(self):
        """
        Setzt die Ampel auf rot.
        """    
        self.ampelphase = "rot"
        self.lampeOben.Einschalten()
        self.lampeMitte.Ausschalten()
        self.lampeUnten.Ausschalten()
    
    

    def Weiterschalten(self):
        """
        Schaltet die Ampelphasen in der Reihenfolge rot, 
        rotgelb, grün, gelb
        """    
        if self.ampelphase == "rot": 
            self.RotgelbSetzen()
        else:
        
            if self.ampelphase == "rotgelb":
                self.GruenSetzen()
            else: 
                if self.ampelphase == "grün":
                    self.GelbSetzen() 
                else:
                    self.RotSetzen()
                
            
        
    


    def AktionAusfuehren(self):
        """
        Schaltet zur gegebenen Zeit die Ampel weiter
        """    
        if self.wartezeit <= 0:
           self.Weiterschalten()
           if self.ampelphase == "rot" or self.ampelphase == "grün":
                self.wartezeit = 30
           else:
                self.wartezeit = 5 
        else:  
            self.wartezeit = self.wartezeit - 1
        
    
    

    def PositionSetzen(self, x, y):
        """
        Setzt die Position der Ampel.
        -- Parameter x x-Position
        -- Parameter y y-Position
        """    
        if x<=800 and x>=0 and y<=600 and y>=0:
            self.lampeOben.PositionSetzen(x, y)
            self.lampeMitte.PositionSetzen(x, y + 20)
            self.lampeUnten.PositionSetzen(x, y + 40)
        
    


    def IstGruen(self):
        """
        Meldet, ob die Ampel auf grün steht.
        -- return True, falls die Ampel grün zeigt.
        """    
        return self.ampelphase == "grün"
    


    def IstGelb(self):
        """
        Meldet, ob die Ampel auf gelb steht.
        -- return True, falls die Ampel gelb zeigt.
        """    
        return self.ampelphase == "gelb"
    



class Fahrbahn(Ereignisbehandlung):
    """
    Verwaltet die Fahrzeuge auf einer Fahrbahn.
    """


    def __init__(self, a, x, y):
        """
        Erzeugt eine leere Fahrbahn ab dem linken Rand unterhalb der Fenstermitte.
        -- Parameter a für diese Richtung zuständige Ampel
        -- Parameter x x-Koordinate der Fahrbahn
        -- Parameter y y-Koordinate der Fahrbahn
        """  
        super().__init__()
        self.x = x
        self.y = y
        self.ampel = a
        self.fahrzeug1 = None
        self.fahrzeug2 = None
        self.darstellung = FahrbahnSymbol()
        self.darstellung.PositionSetzen(self.x, self.y)
        self.start = self.darstellung.EintrittsPositionGeben()
        self.ende = self.darstellung.AustrittsPositionGeben()
        self.rand = self.darstellung.RandAbstandGeben()
        self.wartezeit = 0
    


    def FahrzeugEinreihen(self, fzg):
        """
        Reiht ein neues Fahrzeug an der Startposition in die Fahrbahn ein.
        -- Parameter f das neue Fahrzeug
        """   
        if self.fahrzeug1 == None:
            self.fahrzeug1 = fzg
        elif self.fahrzeug2 == None:
            self.fahrzeug2 = fzg
        fzg.PositionSetzen(self.x + self.rand, self.start)
    


    def IstDraussen(self, fzg):
        """
        Ermittelt, ob ein Fahrzeug entfernt werden muss
        -- Parameter f das in Frage kommende Fahrzeug
        """    
        return fzg.YKoordinateGeben() < self.ende
    


    def VorgaengerZaehlt(self, ampelabstand, vorgaengerabstand, fzg):
        """
        Ermittelt, ob das Fahrzeug sich nach dem Vorgänger richten oder
        die Ampel beachten muss.
        -- Parameter ampelabstand Abstand zur Ampel (Haltelinie):
        -- Parameter vorgaengerabstand Abstand zum Vorgängerfahrzeug
        -- Parameter f das aktuelle Fahrzeug
        -- return True, wenn der Vorgänger wichtig ist, False, wenn die Ampel zählt
        """    
        return (ampelabstand < 0) or (vorgaengerabstand < ampelabstand) or self.ampel.IstGruen() or self.ampel.IstGelb() and (ampelabstand < fzg.GeschwindigkeitGeben()*1.8*5) and (fzg.GeschwindigkeitGeben() > 3.0)
    


    def FahrzeugeBewegen(self):
        """
        Bewegt alle Fahrzeuge der Fahrbahn für eine Sekunde.
        """   
        
        #1. Fahrzeug
        if not (self.fahrzeug1 == None):
            wegmax = 100000
            vvorgaenger = 100
            ampelAbstand = self.fahrzeug1.YKoordinateGeben() - self.darstellung.HaltelinienPositionGeben()
            if self.VorgaengerZaehlt(ampelAbstand, wegmax, self.fahrzeug1):
                self.fahrzeug1.Fahren(wegmax, vvorgaenger)     
            else:      
                self.fahrzeug1.Fahren(ampelAbstand, 0.0)     
            #2. Fahrzeug
            if not (self.fahrzeug2 == None):   
                vvorgaenger = self.fahrzeug1.GeschwindigkeitGeben()
                wegmax = self.fahrzeug2.YKoordinateGeben() - self.fahrzeug1.YKoordinateGeben() - self.fahrzeug1.LaengeGeben() - 2
                if vvorgaenger > 0:   
                    wegmax = wegmax - int((self.fahrzeug2.GeschwindigkeitGeben() + vvorgaenger) / 2.0*1.8*5)         
                ampelAbstand = self.fahrzeug2.YKoordinateGeben() - self.darstellung.HaltelinienPositionGeben()
                if self.VorgaengerZaehlt(ampelAbstand, wegmax, self.fahrzeug2):
                    self.fahrzeug2.Fahren(wegmax, vvorgaenger)
                else:
                    self.fahrzeug2.Fahren(ampelAbstand, 0.0)

        
        #Erstes Fahrzeug draußen?
        if (not (self.fahrzeug1 == None)) and self.IstDraussen(self.fahrzeug1):
            self.fahrzeug1.Entfernen()
            self.fahrzeug1 = self.fahrzeug2
            self.fahrzeug2 = None
        
    
    

    def PositionSetzen(self, x, y):
        """
        Setzt die Position der Fahrbahn.
        -- Parameter x x-Position
        -- Parameter y y-Position
        """    
        self.x = x
        self.y = y
        self.darstellung.PositionSetzen (self.x, self.y)
        self.rand = self.darstellung.RandAbstandGeben()
    
    

    def FahrzeuganzahlGeben(self):                  
        """
        Ausgabe der Fahrzeuganzahl
        -- return Anzahl der Fahrzeuge auf der Fahrbahn
        """   
        if self.fahrzeug1 == None:
            return 0
        elif self.fahrzeug2 == None:
            return 1
        else:
            return 2
       
    


    def AktionAusfuehren(self):
        """
        Fährt die Fahrzeuge und erzeugt gegebenenfalls ein neues Fahrzeug.
        """    
        self.FahrzeugeBewegen()
        if self.wartezeit <= 0:
            self.FahrzeugEinreihen(Fahrzeug())
            self.wartezeit = 4
        else:
            self.wartezeit = self.wartezeit - 1
        
    

  



class Simulation(Ereignisbehandlung):
    """
    Steuert die Simulation
    """

    def __init__(self):
        """
        Legt die Objekte der Simulation an und startet den Taktgeber
        """    
        super().__init__()
        Fahrbahn(Ampel(350, 200), 300, 0)
        self.Starten()
    
    

    # def AktionAusfuehren(self):
    #     """
    #     Nichts, schaltet die Meldung aus.
    #     """
    #     pass

Simulation()