# -- coding: utf-8 --
"""
Created on Wed Apr  8 17:53:04 2020

@author: Klaus Reinold
"""


from graphics_and_games_klassen import Rechteck, Figur, Text, Ereignisbehandlung
from random import randint


class KopfSymbol(Figur):
    """
    Symbol für Kopf
    """
    
    def __init__(self, basisgroesse):
        """
        Inititalisierung des Kopfes
        """
        super().__init__()
        self.basisgroesse = basisgroesse
        self.GroesseSetzen(self.basisgroesse)
        self.FigurteilFestlegenEllipse(-50, -50, 100, 100, "rot")
        self.FigurteilFestlegenEllipse(20,-20,20,40,"hellgruen")
        self.FigurteilFestlegenEllipse(10,-30,20,60,"rot")
        self.FigurteilFestlegenEllipse(-40,-25,40,20,"gelb")
        self.FigurteilFestlegenEllipse(-40,5,40,20,"gelb")
        self.FigurteilFestlegenEllipse(-30,-23,15,10,"schwarz")
        self.FigurteilFestlegenEllipse(-30,9,15,10,"schwarz")   
        self.FigurteilFestlegenDreieck(-50,-50,-35,-35,-40,-35,"schwarz")
        self.FigurteilFestlegenDreieck(-50,50,-35,35,-40,35,"schwarz")
        self.xPos = 0
        self.yPos = 0
        self.winkel = 0


        self.PositionSetzen(self.xPos, self.xPos)

        
    
    def PositionSetzen(self, x, y):
        """
        Setzt die Position des Kopfelements.Der Ursprung liegt links oben
        Fensters, die y-Achse zeigt nach unten.(x /y) bedeutet das
        Kästchen rechts unterhalb der Gitterlinien.
        -- Parameter x x-Position
        -- Parameter y y-Position
        """
        self.xPos = x
        self.yPos = y
        super().PositionSetzen(self.xPos * self.basisgroesse+self.basisgroesse/2, self.yPos * self.basisgroesse+self.basisgroesse/2)
    
    def XPositionGeben(self):
        """
        gibt die x-Koordinate zurück
        """
        return self.xPos

    def YPositionGeben(self):
        """
        gibt die y-Koordinate zurück
        """
        return self.yPos
    

class RumpfSymbol(Figur):
    """
    Symbol für Rumpf
    """
    
    def __init__(self, basisgroesse):
        """
        Inititalisierung des Rumpfes
        """
        super().__init__()
        self.basisgroesse = basisgroesse
        self.GroesseSetzen(int(self.basisgroesse))
        self.FigurteilFestlegenEllipse(-50,-50,60,100,"hellgruen")
        self.FigurteilFestlegenEllipse(-10,-50,60,100,"gruen")
        self.xPos = 0
        self.yPos = 0
        self.PositionSetzen(self.xPos, self.yPos)

        
    
    def PositionSetzen(self, x, y):
        """
        Setzt die Position des Kopfelements.Der Ursprung liegt links oben
        Fensters, die y-Achse zeigt nach unten.(x /y) bedeutet das
        Kästchen rechts unterhalb der Gitterlinien.
        -- Parameter x x-Position
        -- Parameter y y-Position
        """
        self.xPos = x
        self.yPos = y
        super().PositionSetzen(self.xPos * self.basisgroesse+self.basisgroesse/2, self.yPos * self.basisgroesse+self.basisgroesse/2)
    
    def XPositionGeben(self):
        """
        gibt die x-Koordinate zurück
        """
        return self.xPos

    def YPositionGeben(self):
        """
        gibt die y-Koordinate zurück
        """
        return self.yPos

        
class Schlange:
    """
    verwaltet die Schlange
    """
    
    def __init__(self, x, y, startwinkel, basisgroesse):
        """
        Initialisierung
        -- Parameter x x-Wert der Rasterkoordinaten
        -- Parameter y y-Wert der Rasterkoordinaten
        -- Winkel Startwinkel (0, 90, 180, 270)
        """
        self.kopf = KopfSymbol(basisgroesse)
        self.kopf.WinkelSetzen(startwinkel)
        self.kopf.PositionSetzen(x, y)
        self.winkel = startwinkel
        self.rumpf1 = RumpfSymbol(basisgroesse)
        self.rumpf2 = RumpfSymbol(basisgroesse)
        if self.winkel==90 or self.winkel == 270:
            self.rumpf1.Drehen(90)
            self.rumpf2.Drehen(90)
        self.basisgroesse = basisgroesse
        if startwinkel == 0:
            self.rumpf2.PositionSetzen(x-1, y)
            self.rumpf1.PositionSetzen(x-2, y)
        elif startwinkel == 90:
            self.rumpf2.PositionSetzen(x, y+1)
            self.rumpf1.PositionSetzen(x, y+2)
        elif startwinkel == 180:
            self.rumpf2.PositionSetzen(x+1, y)
            self.rumpf1.PositionSetzen(x+2, y) 
        elif startwinkel == 270:
            self.rumpf2.PositionSetzen(x, y-1)
            self.rumpf1.PositionSetzen(x, y-2)

        
    
    def Bewegen(self):
        """
        Bewegt die Schlange ein Feld in der eingestellten Richtung.
        """
        x = self.kopf.XPositionGeben()
        y = self.kopf.YPositionGeben()
        xalt = x
        yalt = y
        if self.kopf.WinkelGeben() == 0:
            x = x + 1
        elif self.kopf.WinkelGeben() == 90:
            y = y - 1
        elif self.kopf.WinkelGeben() == 180:
            x = x - 1
        elif self.kopf.WinkelGeben() == 270:
            y = y + 1
        
        self.kopf.PositionSetzen(x, y)
        self.rumpf1.Entfernen()
        self.rumpf1 = self.rumpf2
        self.rumpf2 = RumpfSymbol(self.basisgroesse)
        if self.winkel==90 or self.winkel == 270:
            self.rumpf2.Drehen(90)
        self.rumpf2.PositionSetzen(xalt, yalt)
    
    



    def XPositionGeben(self):
        """
        Gibt die x-Position des Schlangenkopfs zurück.
        -- return x-Position des Schlangenkopfs
        """
        return self.kopf.XPositionGeben()
    
    
    def YPositionGeben(self):
        """
        Gibt die x-Position des Schlangenkopfs zurück.
        -- return x-Position des Schlangenkopfs
        """
        return self.kopf.YPositionGeben()
    
        

    def KopfInRumpf(self):
        """
        Prueft ob der Schlangenkopf auf der Position eines Rumpfsymbols ist, d.h.sich die Schlange selbst gebissen hat.
        return wahr, wenn der Kopf im Rumpf liegt.
        """
        return self.kopf.BeruehrtFarbe("gruen")    
    
    def WinkelSetzen(self, winkel):
        """
        Setzt den Winkel
        """
        self.winkel = winkel
        self.kopf.WinkelSetzen(winkel)
        
 
            

class Spiel(Ereignisbehandlung):
    """
    Schafft den Rahmen fuer das Spiel.
    """
    

    def __init__(self):  
        """
        Legt das Spiel an
        """   
        super().__init__()

        r = Rechteck()
        r.FarbeSetzen("grau")
        r.GroesseSetzen(672, 504)
        r.PositionSetzen(0,0)
        self.basisGroesse = 28
        self.anzahlKaestchenBreite = 24
        self.anzahlKaestchenHoehe = 18
        self.zellen = [[None]*self.anzahlKaestchenBreite for i in range (self.anzahlKaestchenHoehe)]
        
        for i in range (0, 18):
            for j in range (0, 24):
                self.zellen[i][j] = Rechteck()
                self.zellen[i][j].FarbeSetzen("hellgelb")
                self.zellen[i][j].PositionSetzen(j*self.basisGroesse, i*self.basisGroesse)
                self.zellen[i][j].GroesseSetzen(self.basisGroesse-1, self.basisGroesse-1)
        
        xStart =  randint(0, 20)
        yStart = randint(0, 17)
        richtung = randint(0, 3)
        
        self.schlange = Schlange(xStart, yStart, richtung*90, self.basisGroesse)
        
        self.punkte = 0
        self.punkteanzeige = Text()
        self.punkteanzeige.PositionSetzen(700, 30)
        self.punkteanzeige.TextSetzen("Punkte: "+str(self.punkte))
        
        self.TaktdauerSetzen(500)



    def TasteGedrueckt(self, taste):  
        """
        Reaktion auf Tastaturereignisse
        """ 
        if taste == "s" or taste == "S":
            self.Starten()
        elif taste == "p" or taste == "P":
            self.Anhalten()
        elif taste == "down":
            self.schlange.WinkelSetzen(270)
        elif taste == "left":
            self.schlange.WinkelSetzen(180)
        elif taste == "right":
            self.schlange.WinkelSetzen(0)
        elif taste == "up":
            self.schlange.WinkelSetzen(90)
        


    def AktionAusfuehren(self):  
        """
        Methode, die vom Taktgeber regelmäßig aufgerufen wird
        """   
        if self.schlange.XPositionGeben()<self.anzahlKaestchenBreite and self.schlange.XPositionGeben()>= 0 and  self.schlange.YPositionGeben()<self.anzahlKaestchenHoehe and self.schlange.YPositionGeben()>= 0 and not self.schlange.KopfInRumpf():
            self.schlange.Bewegen()
            self.punkte += 1
            self.punkteanzeige.TextSetzen("Punkte: "+str(self.punkte))
        else:
            self.Anhalten()
            t = Text(textgroesse = 20)
            if self.schlange.KopfInRumpf():  
                t.TextSetzen("Game over - Schlange hat sich selbst gebissen")
            else:
                t.TextSetzen("Game over - Spielfeld verlassen")
            
Spiel()
