# -*- coding: utf-8 -*-
"""
Created on Fry Jan 1 12:33:57 2021

@author: Johannes Neumeyer, Florian Janus, Albert Wiedemann
"""

from graphics_and_games_klassen import Rechteck, Kreis, Dreieck, Text


class Windsensor:
    # Legt den Namen und den Beobachter fest
    def __init__(self, namensZusatz):
        self.name = "Windsensor" + namensZusatz
        self.beobachter = None
        self.aktuellerWert = 0

    # Prüft, ob die Eingabe akzeptiert werden kann
    def EingabePruefen(self, eingabe):
        try:
            wert = int(eingabe)
            return 0 <= wert <= 12
        except:
            return False

    # Registriert einen Beobachter
    # param beobachter wird von jeder Wertänderung benachrichtigt
    def BeobachterRegistrieren(self, beobachter):
        self.beobachter = beobachter

    # Setzt den neuen Sensorwert
    def WertSetzen(self, eingabe):
        wert = int(eingabe)
        if self.aktuellerWert != wert:
            self.aktuellerWert = wert
            self.beobachter.WertaenderungMeldenWindsensor(self.aktuellerWert)

    # Meldet den Zustand des Sensors
    def ZustandGeben(self):
        return str(self.aktuellerWert) + " Bft"


class Uhr:
    # Legt den Namen und den Beobachter fest
    def __init__(self, namensZusatz):
        self.name = "Uhr" + namensZusatz
        self.stunden = 0
        self.minuten = 0
        self.sekunden = 0
        self.beobachter = None

    # Registriert einen Beobachter
    # param beobachter wird von jeder Wertänderung benachrichtigt
    def BeobachterRegistrieren(self, beobachter):
        self.beobachter = beobachter

    # Setzt den neuen Sensorwert
    def WertSetzen(self, eingabe):
        if eingabe >= 0 and eingabe <=24:
            self.stunden = eingabe
            self.beobachter.WertaenderungMeldenUhr(self.stunden * 10000 + self.minuten*100 + self.sekunden)

    # Meldet den Zustand der Uhr
    def ZustandGeben(self):
        if self.minuten < 10:
            minutenString = "0" + str(self.minuten)
        else:
            minutenString = str(self.minuten)
        if self.sekunden < 10:
            sekundenString = "0" + str(self.sekunden)
        else:
            sekundenString = str(self.sekunden)
        return str(self.stunden) + ":" + minutenString + ":" + sekundenString + " Uhr"


class Thermometer:
    # Legt den Namen und den Beobachter fest
    def __init__(self, namensZusatz):
        self.name = "Thermometer" + namensZusatz
        self.aktuellerWert = 0
        self.beobachter = []

    # Registriert einen Beobachter
    # param beobachter wird von jeder Wertänderung benachrichtigt
    def BeobachterRegistrieren(self, beobachter):
        self.beobachter.append(beobachter)

    # Prüft, on doe Eingabe akzeptiert werden kann
    def EingabePruefen(self, eingabe):
        try:
            wert = int(eingabe)
            return -50 <= wert <= 100
        except:
            return False

    # Setzt  den neuen Sensorwert
    def WertSetzen(self, eingabe):
        wert = int(eingabe)
        if self.aktuellerWert != wert:
            if self.EingabePruefen(wert):
                self.aktuellerWert = wert
                for b in self.beobachter:
                    b.WertaenderungMelden(self.aktuellerWert)

    # Meldet den Zustand des Sensors
    def ZustandGeben(self):
        return str(self.aktuellerWert) + " °C"


class Bewegungsmelder:
    # Legt den Namen und den Beobachter fest.
    def __init__(self, namensZusatz):
        self.name = "Bewegungsmelder" + namensZusatz
        self.an = False
        self.erlaubt = {"an": True, "ein": True, "wahr": True,
                        "true": True, "aus": False, "falsch": False, "false": False}
        self.beobachter = None#

    # Registriert einen Beobachter
    # param beobachter wird von jeder Wertänderung benachrichtigt
    def BeobachterRegistrieren(self, beobachter):
        self.beobachter = beobachter

    # Prüft, ob die Eingabe akzeptiert werden kann
    def EingabePruefen(self, eingabe):
        e = eingabe.lower()
        if e in self.erlaubt:
            return True
        else:
            return False

    # Setzt den neuen Sensorwert
    def WertSetzen(self, eingabe):
        self.an = eingabe
        self.beobachter.WertaenderungMeldenBewegungsmelder(self.an)

    # Meldet den Zustand des Sensors
    def ZustandGeben(self):
        if self.an:
            return "Bewegung"
        else:
            return "Ruhe"


class Sonnensensor:
    # Legt den Namen und den Beobachter fest.
    def __init__(self, namensZusatz):
        self.name = "Sonnensensor" + namensZusatz
        self.aktuellerWert = 0
        self.beobachter = None

    # Registriert einen Beobachter
    # param beobachter wird von jeder Wertänderung benachrichtigt
    def BeobachterRegistrieren(self, beobachter):
        self.beobachter = beobachter

    # Prüft, ob die Eingabe akzeptiert werden kann
    def EingabePruefen(self, eingabe):
        try:
            wert = int(eingabe)
            return 0 <= wert <= 100
        except:
            return False

    # Setzt den neuen Sensorwert
    def WertSetzen(self, eingabe):
        wert = int(eingabe)
        if self.aktuellerWert != wert:
            if self.EingabePruefen(wert):
                self.aktuellerWert = wert
                self.beobachter.WertaenderungMeldenSonnensensor(self.aktuellerWert)

    # Meldet den Zustand des Sensors
    def ZustandGeben(self):
        return str(self.aktuellerWert) + " %"


class Waage:
    # Legt den Namen und den Beobachter fest.
    def __init__(self, namensZusatz):
        self.name = "Waage " + namensZusatz
        self.aktuellerWert = 0
        self.beobachter = []

    # Registriert einen Beobachter
    # param beobachter wird von jeder Wertänderung benachrichtigt
    def BeobachterRegistrieren(self, beobachter):
        self.beobachter.append(beobachter)

    # Prüft, ob die Eingabe akzeptiert werden kann
    def EingabePruefen(self, eingabe):
        try:
            wert = int(eingabe)
            return 0 <= wert <= 10000
        except:
            return False

    # Setzt den neuen Sensorwert
    def WertSetzen(self, eingabe):
        wert = int(eingabe)
        if self.aktuellerWert != wert:
            self.aktuellerWert = wert
            for b in self.beobachter:
                b.WertaenderungMelden(self.aktuellerWert)

    # Meldet den Zustand des Sensors
    def ZustandGeben(self):
        return str(self.aktuellerWert) + " g"


class LampenSymbol:
    # Baut das Symbol auf
    def __init__(self, x, y):
        self.kabel = Rechteck()
        self.kabel.PositionSetzen(x, y)
        self.kabel.GroesseSetzen(1, 30)
        self.kabel.FarbeSetzen("schwarz")
        self.birne = Kreis()
        self.birne.RadiusSetzen(10)
        self.birne.PositionSetzen(x, y + 55)
        self.birne.FarbeSetzen("weiss")
        self.schirm = Dreieck()
        self.schirm.GroesseSetzen(50, 25)
        self.schirm.PositionSetzen(x, y + 30)
        self.schirm.FarbeSetzen("grau")

    # Meldet den neuen Zustand an das Symbol
    def AenderungMelden(self, zustandNeu):
        if zustandNeu == True:
            self.birne.FarbeSetzen("gelb")
        else:
            self.birne.FarbeSetzen("weiss")


class RollladenSymbol:
    # Baut das Symbol auf
    def __init__(self, x, y):
        self.lamelle0 = Rechteck()
        self.lamelle0.FarbeSetzen("schwarz")
        self.lamelle0.GroesseSetzen(100, 10)
        self.lamelle0.PositionSetzen(x, y)

        self.lamelle1 = Rechteck()
        self.lamelle1.FarbeSetzen("schwarz")
        self.lamelle1.GroesseSetzen(100, 10)
        self.lamelle1.PositionSetzen(x, y + 11)

        self.lamelle2 = Rechteck()
        self.lamelle2.FarbeSetzen("schwarz")
        self.lamelle2.GroesseSetzen(100, 10)
        self.lamelle2.PositionSetzen(x, y + 22)

        self.lamelle3 = Rechteck()
        self.lamelle3.FarbeSetzen("schwarz")
        self.lamelle3.GroesseSetzen(100, 10)
        self.lamelle3.PositionSetzen(x, y + 33)

        self.lamelle4 = Rechteck()
        self.lamelle4.FarbeSetzen("schwarz")
        self.lamelle4.GroesseSetzen(100, 10)
        self.lamelle4.PositionSetzen(x, y + 44)

        self.lamelle5 = Rechteck()
        self.lamelle5.FarbeSetzen("schwarz")
        self.lamelle5.GroesseSetzen(100, 10)
        self.lamelle5.PositionSetzen(x, y + 55)

        self.lamelle6 = Rechteck()
        self.lamelle6.FarbeSetzen("schwarz")
        self.lamelle6.GroesseSetzen(100, 10)
        self.lamelle6.PositionSetzen(x, y + 66)

        self.lamelle7 = Rechteck()
        self.lamelle7.FarbeSetzen("schwarz")
        self.lamelle7.GroesseSetzen(100, 10)
        self.lamelle7.PositionSetzen(x, y+77)

        self.lamelle8 = Rechteck()
        self.lamelle8.FarbeSetzen("schwarz")
        self.lamelle8.GroesseSetzen(100, 10)
        self.lamelle8.PositionSetzen(x, y+88)

    # Meldet den neuen Zustand an das Symbol
    def AenderungMelden(self, zustandNeu):
        if zustandNeu:
            self.lamelle1.SichtbarkeitSetzen(False)
            self.lamelle2.SichtbarkeitSetzen(False)
            self.lamelle3.SichtbarkeitSetzen(False)
            self.lamelle4.SichtbarkeitSetzen(False)
            self.lamelle5.SichtbarkeitSetzen(False)
            self.lamelle6.SichtbarkeitSetzen(False)
            self.lamelle7.SichtbarkeitSetzen(False)
            self.lamelle8.SichtbarkeitSetzen(False)
        else:
            self.lamelle1.SichtbarkeitSetzen(True)
            self.lamelle2.SichtbarkeitSetzen(True)
            self.lamelle3.SichtbarkeitSetzen(True)
            self.lamelle4.SichtbarkeitSetzen(True)
            self.lamelle5.SichtbarkeitSetzen(True)
            self.lamelle6.SichtbarkeitSetzen(True)
            self.lamelle7.SichtbarkeitSetzen(True)
            self.lamelle8.SichtbarkeitSetzen(True)


class HeizkoerperSymbol:
    # Baut das Symbol auf
    def __init__(self, x, y):
        self.innenkreise = []
        self.innenrechtecke = []
        for index in range(0, 8):
            self.HeizkoerperSegmentAnPos(x + index * 10, y)

    def HeizkoerperSegmentAnPos(self, x, y):
        # Segment außen
        oben = Kreis()
        oben.RadiusSetzen(10)
        oben.PositionSetzen(x + 10, y + 10)
        oben.FarbeSetzen("schwarz")

        mitte = Rechteck()
        mitte.GroesseSetzen(21, 40)
        mitte.PositionSetzen(x, y + 10)
        mitte.FarbeSetzen("schwarz")

        unten = Kreis()
        unten.RadiusSetzen(10)
        unten.PositionSetzen(x + 10, y + 50)
        unten.FarbeSetzen("schwarz")

        # Segment innen
        obenInnen = Kreis()
        obenInnen.RadiusSetzen(8)
        obenInnen.PositionSetzen(x + 10, y + 10)
        obenInnen.FarbeSetzen("weiss")
        self.innenkreise.append(obenInnen)

        mitteInnen = Rechteck()
        mitteInnen.GroesseSetzen(17, 42)
        mitteInnen.PositionSetzen(x + 2, y + 9)
        mitteInnen.FarbeSetzen("weiss")
        self.innenrechtecke.append(mitteInnen)

        untenInnen = Kreis()
        untenInnen.RadiusSetzen(8)
        untenInnen.PositionSetzen(x+10, y + 50)
        untenInnen.FarbeSetzen("weiss")
        self.innenkreise.append(untenInnen)

    # Meldet den neuen Zustand an das Symbol
    def AenderungMelden(self, zustandNeu):
        if zustandNeu == True:
            for k in self.innenkreise:
                k.FarbeSetzen("rot")
            for r in self.innenrechtecke:
                r.FarbeSetzen("rot")
        else:
            for k in self.innenkreise:
                k.FarbeSetzen("weiss")
            for r in self.innenrechtecke:
                r.FarbeSetzen("weiss")

class FensterSymbol:
    # Erzeugt das Fenstersymbol
    def __init__(self, x, y):
        hoehe = 100
        breite = hoehe

        self.links = Rechteck()
        self.links.GroesseSetzen(2, hoehe)
        self.links.PositionSetzen(x, y)
        self.links.FarbeSetzen("braun")

        self.oben = Rechteck()
        self.oben.GroesseSetzen(breite, 2)
        self.oben.PositionSetzen(x, y)
        self.oben.FarbeSetzen("braun")

        self.unten = Rechteck()
        self.unten.GroesseSetzen(breite, 2)
        self.unten.PositionSetzen(x, y + hoehe - 2)
        self.unten.FarbeSetzen("braun")

        self.rechts = Rechteck()
        self.rechts.GroesseSetzen(2, hoehe)
        self.rechts.PositionSetzen(x + breite - 2, y)
        self.rechts.FarbeSetzen("braun")

        self.kreuzSenkrecht = Rechteck()
        self.kreuzSenkrecht.GroesseSetzen(2, hoehe)
        self.kreuzSenkrecht.PositionSetzen(x + breite / 2, y)
        self.kreuzSenkrecht.FarbeSetzen("braun")

        self.kreuzWaagrecht = Rechteck()
        self.kreuzWaagrecht.GroesseSetzen(breite, 2)
        self.kreuzWaagrecht.PositionSetzen(x, y + hoehe / 2)
        self.kreuzWaagrecht.FarbeSetzen("braun")


class Hausmauer(Rechteck):
    def __init__(self, x=None, y=None):
        super().__init__()
        self.FarbeSetzen("grau")
        if x is not None and y is not None:
            self.PositionSetzen(x, y)


class HintergrundSymbol:
    # Legt die Wände, das Dach und die Beschriftung an
    def __init__(self):
        dachlinks = Dreieck()
        dachlinks.FarbeSetzen("rot")
        dachlinks.GroesseSetzen(550, 100)
        dachlinks.PositionSetzen(400, 0)
        unten = Hausmauer(150, 500)
        unten.GroesseSetzen(500, 5)
        mitteWaagrecht = Hausmauer(150, 300)
        mitteWaagrecht.GroesseSetzen(500, 5)
        oben = Hausmauer(150, 100)
        oben.GroesseSetzen(500, 5)
        links = Hausmauer(150, 100)
        links.GroesseSetzen(5, 400)
        rechts = Hausmauer(645, 100)
        rechts.GroesseSetzen(5, 400)
        mitteSenkrecht = Hausmauer(395, 100)
        mitteSenkrecht.GroesseSetzen(5, 400)
        FensterSymbol(250, 125)
        FensterSymbol(250, 325)
        FensterSymbol(500, 125)
        FensterSymbol(500, 325)

        wohnzimmer1 = Text()
        wohnzimmer1.TextSetzen("Wohnzimmer1")
        wohnzimmer1.FarbeSetzen("grau")
        wohnzimmer1.TextGroesseSetzen(15)
        wohnzimmer1.PositionSetzen(160, 120)

        wohnzimmer2 = Text()
        wohnzimmer2.TextSetzen("Wohnzimmer2")
        wohnzimmer2.FarbeSetzen("grau")
        wohnzimmer2.TextGroesseSetzen(15)
        wohnzimmer2.PositionSetzen(405, 120)

        kueche = Text()
        kueche.TextSetzen("Küche")
        kueche.FarbeSetzen("grau")
        kueche.TextGroesseSetzen(15)
        kueche.PositionSetzen(160, 320)

        esszimmer = Text()
        esszimmer.TextSetzen("Esszimmer1")
        esszimmer.FarbeSetzen("grau")
        esszimmer.TextGroesseSetzen(15)
        esszimmer.PositionSetzen(405, 320)


class Lampe:
    # Legt den Namen und das Darstellungssymbol fest
    def __init__(self, namensZusatz, lampensymbol, bewegungsmelder, sonnensensor):
        self.name = "Lampe" + namensZusatz
        self.bewegung = False
        self.helligkeit = 0
        self.an = False
        self.automatik = True
        self.symbol = lampensymbol
        self.symbol.AenderungMelden(self.an)
        bewegungsmelder.BeobachterRegistrieren(self)
        sonnensensor.BeobachterRegistrieren(self)

    # Wird vom Sonnensensor bei einem neuen Helligkeitswert aufgerufen
    def WertaenderungMeldenSonnensensor(self, wert):
        self.helligkeit = wert
        if self.automatik:
            self.an = self.bewegung and (self.helligkeit < 30)
            self.symbol.AenderungMelden(self.an)

    # Wird von Bewegungsmelder bei einem neuen Bewegungszustand aufgerufen
    # aram wert der neue Bewegungszustand
    def WertaenderungMeldenBewegungsmelder(self, wert):
        self.bewegung = wert
        if self.automatik:
            self.an = self.bewegung and (self.helligkeit < 30)
            self.symbol.AenderungMelden(self.an)

    # Meldet den Zustand der Komponente
    def ZustandGeben(self):
        if self.an:
            return "ein"
        else:
            return "aus"

    # Schaltet die Komponente dauerhaft ein
    def Einschalten(self):
        self.an = True
        self.automatik = False
        self.symbol.AenderungMelden(self.an)

    # Schaltet die Komponente dauerhaft aus
    def Ausschalten(self):
        self.an = False
        self.automatik = False
        self.symbol.AenderungMelden(self.an)

    # Schaltet die Komponente auf Automatikbetrieb
    def AutomatikAktivieren(self):
        self.an = self.bewegung and (self.helligkeit < 30)
        self.automatik = True
        self.symbol.AenderungMelden(self.an)


class Rollladen:
    # Legt den Namen und das Darstellungssymbol fest
    def __init__(self, namensZusatz, symbol, windsensor, uhr):
        self.name = "Rolladen" + namensZusatz
        self.oben = False
        self.automatik = True
        windsensor.BeobachterRegistrieren(self)
        uhr.BeobachterRegistrieren(self)
        self.symbol = symbol
        self.windsensor = windsensor
        self.uhr = uhr
        self.windwert = 0
        self.uhrzeit = 0

    # Wird vom Windsensor bei einer neuen Windstärke aufgerufen
    # param wert die neue Windstärke
    def WertaenderungMeldenWindsensor(self, wert):
        self.windwert = wert
        if self.automatik:
            self.oben = (self.windwert > 6) or (self.uhrzeit > 60000) and (self.uhrzeit < 200000)
            self.symbol.AenderungMelden(self.oben)

    # Wird von der Uhr bei einer neuen Uhrzeit aufgerufen
    # param wert die neue Uhrzeit
    def WertaenderungMeldenUhr(self, wert):
        self.uhrzeit = wert
        if self.automatik:
            self.oben = (self.windwert > 6) or (self.uhrzeit > 60000) and (self.uhrzeit < 200000)
            self.symbol.AenderungMelden(self.oben)

    # Meldet den Zustand der Komponente
    def ZustandGeben(self):
        if self.oben:
            return "offen"
        else:
            return "geschlossen"

    # Schaltet die Komponente dauerhaft ein
    def Runterlassen(self):
        self.oben = False
        self.automatik = False
        self.symbol.AenderungMelden(self.oben)

    # Schaltet die Komponente dauerhaft aus
    def Hochziehen(self):
        self.oben = True
        self.automatik = False
        self.symbol.AenderungMelden(self.oben)

    # Schaltet die Komponente auf Automatikbetrieb
    def AutomatikAktivieren(self):
        self.oben = (self.windwert > 6) or (self.uhrzeit > 60000) and (self.uhrzeit < 200000)
        self.automatik = True
        self.symbol.AenderungMelden(self.oben)


class Heizkoerper:
    # Legt den Namen und das Darstellungssymbol fest
    def __init__(self, namensZusatz, symbol, thermometer):
        self.name = "Heizkörper" + namensZusatz
        self.temperatur = 0
        self.an = True
        self.automatik = True
        thermometer.BeobachterRegistrieren(self)
        self.symbol = symbol
        self.symbol.AenderungMelden(self.an)

    # Wird vom Thermometer bei einer neuen Temperatur aufgerufen
    def WertaenderungMelden(self, wert):
        self.temperatur = wert
        if self.automatik:
            self.an = self.temperatur < 22
            self.symbol.AenderungMelden(self.an)

    # Meldet den Zustand der Komponente
    def ZustandGeben(self):
        if self.an:
            return "ein"
        else:
            return "aus"

    # Schaltet die Komponente dauerhaft ein
    def Einschalten(self):
        self.an = True
        self.automatik = False
        self.symbol.AenderungMelden(self.an)

        # Schaltet die Komponente dauerhaft aus

    def Ausschalten(self):
        self.an = False
        self.automatik = False
        self.symbol.AenderungMelden(self.an)

    # Schaltet die Komponente auf Automatikbetrieb
    def AutomatikAktivieren(self):
        self.an = self.temperatur < 22
        self.automatik = True
        self.symbol.AenderungMelden(self.an)

class Kuehlschrank:
    # Legt den Namen und das Darstellungssymbol fest
    def __init__(self, namensZusatz, sy, waage):
        self.name = "Kühlschrank" + namensZusatz
        self.gewicht = 0
        self.leer = True
        self.symbol = sy
        self.symbol.AenderungMelden(self.leer)
        waage.BeobachterRegistrieren(self)

    # Wird von der Waage bei einem neuen Gewicht aufgerufen
    def WertaenderungMelden(self, wert):
        self.gewicht = wert
        self.leer = self.gewicht < 100
        self.symbol.AenderungMelden(self.leer)

    # Meldet den Zustand der Komponente
    def ZustandGeben(self):
        if self.leer:
            return "Eierfach leer"
        else:
            return "Eierfach voll"

    # Tut nichts
    def Einschalten(self):
        pass

    # Tut nichts
    def Ausschalten(self):
        pass

    # Tut nichts
    def AutomatikAktivieren(self):
        pass


class Haus:
    def __init__(self):
        self.hintergrund = HintergrundSymbol()
        self.thermometerEsszimmer = Thermometer("Esszimmer")
        self.heizkoerperSymbolEsszimmer = HeizkoerperSymbol(420, 440)
        self.heizkoerperEsszimmer = Heizkoerper("Esszimmer", self.heizkoerperSymbolEsszimmer, self.thermometerEsszimmer)

    # Setzt das Thermometer im Esszimmer auf einen neuen Wert
    # param eingabe neuer Temperaturwert
    def ThermometerEsszimmerWertSetzen (self, eingabe):
        self.thermometerEsszimmer.WertSetzen(eingabe)

    # Schaltet den Heizkörper im Esszimmer dauerhaft ein
    def HeizkoerperEsszimmerEinschalten(self):
        self.heizkoerperEsszimmer.Einschalten()

    # Schaltet den Heizkörper im Esszimmer dauerhaft aus
    def HeizkoerperEsszimmerAusschalten(self):
        self.heizkoerperEsszimmer.Ausschalten()

    # Setzt den Heizkörper im Esszimmer auf Automatikbetrieb
    def HeizkoerperEsszimmerAutomatikSetzen(self):
        self.heizkoerperEsszimmer.AutomatikAktivieren()

    # Zeigt die Zustände aller Geräte an
    def ZustaendeAnzeigen(self):
        print(self.thermometerEsszimmer.name + ": " + self.thermometerEsszimmer.ZustandGeben())
        print(self.heizkoerperEsszimmer.name + ": " + self.heizkoerperEsszimmer.ZustandGeben())


h = Haus()
